function [CBl,CBu] = LUBounds(adjmat,grpind)

% Function to compute lower and upper bounds for group betweenness B(A),
% for arbitrary group size m=|A|.  Uses an exhaustive algorithm to select
% an optimal ordering (i1,...,im) for the m vertices, in constructing 
% the upper boundi based on the pairs (i1,i2), (i2,i3), ..., (im-1,im).
%
% NOTE: Calls the 'perm' algorithm in Matlab, which is not recommended
%       for input of more than 15.  So this code should be applied
%       only to m relatively small (i.e., certainly less than 15).
%
% Input
% adjmat := adjacency matrix
% grpind := mx1 vector of indices of group members.
%
% Output
% lubnds = 2x1 vector with lower and upper bounds.
%
% Calls:  compute_CoB_res_nodes.m


resCoB = compute_CoB_res_nodes(adjmat,'full',grpind);
Cm = resCoB(grpind,grpind);

% Construct lower bound.
CBl = sum(diag(Cm)) - sum(sum(tril(Cm,-1)));

% Construct upper bound.
% Search for best linear ordering of vertices in group.
mm = length(grpind);
pmat = perms([1:mm]);
nperms = size(pmat,1);
CandUpperBnds = zeros(nperms,1);

for i=1:nperms,
  temp = 0;
  for j=1:mm-1,
    temp = temp + Cm(pmat(i,j),pmat(i,j+1));
  end
  CandUpperBnds(i) = temp;
end
  
best = CandUpperBnds(CandUpperBnds==max(CandUpperBnds));
CBu = sum(diag(Cm)) - best(1);

%  This code has been developed by David Chua, Eric Kolaczyk, and
%  Mark Barthelemy.  See Kolaczyk, Chua, and Barthelemy (2009),
%  Group-betweenness and cobetweenness: Inter-related notions of coalition
%  centrality.  Social Networks, 31:3, 190-203.

%
%  Copyright (c) David Chua, Eric Kolaczyk,and Marc Barthelemy, 2009.
%  All rights reserved.
