function [CoB] = compute_CoB_res_nodes(A, mat_type, CoalInd)
%
% compute_CoB_res_nodes(A, mat_type, CoalInd) 
% takes the adjacency matrix for an unweighted
% graph and computes the co-betweenness matrix under the assumption
% that all shorest paths between two nodes are equiprobable.  However, in
% adding up path contributions, it only does so for those nodes outside the
% vector CoalInd.
%
% A(i,j) = 1 if edge (i,j) exists, zero otherwise
% mat_type = optional argument, 'full' uses full matrices for CoB,
%            and 'sparse' uses sparse matrices for CoB. Default
%            is the faster (but space hungry) full matrix option.
% CoB(i,j) = co-betweenness of nodes i and j. 
% CoalInd = vector of nodes in the coalition to be excluded.
%
% Warnings: 1. Uses the global variable CoB_global
%           2. CoB_global assumes sparsity < 12%, will slow down
%              considerably if this is exceeded. 

% CVS version control block - do not edit manually
%  $RCSfile: compute_CoB_nodes.m,v $
%  $Author: dchua $
%  $Revision: 1.3 $
%  $Date: 2005/09/27 23:02:06 $
%  $Source: /home/dchua/.cvs/CoB/compute_CoB_nodes.m,v $
% End of CVS version control block

nNodes = size(A,1);
global CoB_global;
if nargin < 2
  mat_type = 'full';
end
if strcmp(lower(mat_type), 'full')
  CoB_global = zeros(nNodes);
else
  CoB_global = sparse([], [], [], nNodes, nNodes, round(nNodes^2 * 0.20));
end

% Set up vector of non-coalition indices.
indvec = [1:nNodes];
for i=1:length(CoalInd),
  indvec = indvec(indvec~=CoalInd(i));
end

% Main loop over source nodes s.
for s = indvec
  [d, nPaths, verticesByDist, child, parent] = ...
      BF_count_paths(s, A, nNodes);
  [score] = ...
      score_res_nodes(s,verticesByDist,child,parent,nPaths,nNodes,CoalInd);
  DF_visit_nodes(s, s, [], child, score, nPaths);
end

CoB = CoB_global + CoB_global' - diag(diag(CoB_global));
clear global CoB_global;

%  This code has been developed by David Chua, Eric Kolaczyk, and
%  Mark Barthelemy.  See Kolaczyk, Chua, and Barthelemy (2009),
%  Group-betweenness and cobetweenness: Inter-related notions of coalition 
%  centrality.  Social Networks, 31:3, 190-203.

%
%  Copyright (c) David Chua, Eric Kolaczyk,and Marc Barthelemy, 2009.
%  All rights reserved.
